from cyares import Channel
from cyares.channel import CYARES_SOCKET_BAD
from wepoll import EpollSelector
from wepoll import epoll, EPOLLIN, EPOLLOUT
from socket import AF_INET
READ = EPOLLIN
WRITE = EPOLLOUT

# based off pycares's testsuite

class TestCyaresWepoll:
    channel: Channel
    def wait(self):
        # The function were really testing is this wait function
        poll = epoll()
        while True:
            r, w = self.channel.getsock()
            if not r and not w:
                break
            for rs in r:
                poll.register(rs, EPOLLIN)
            for ws in w:
                poll.register(ws, EPOLLOUT)

            timeout = self.channel.timeout()
            if timeout == 0.0:
                self.channel.process_fd(
                    CYARES_SOCKET_BAD, CYARES_SOCKET_BAD
                )
                continue
            for fd, event in poll.poll(timeout):
                if event & ~EPOLLIN:
                    self.channel.process_write_fd(fd)
                if event & ~EPOLLOUT:
                    self.channel.process_read_fd(fd)

    def test_resolve(self):
        self.channel = Channel(event_thread=False, servers=["8.8.8.8", "8.8.4.4"])
        fut = self.channel.gethostbyname("python.org", AF_INET)
        self.wait()
        self.channel.cancel()
        assert fut.result()





from wepoll import WepollEventLoop
from cyares.aio import DNSResolver
import pytest

import anyio


@pytest.fixture(
    params=[pytest.param(("asyncio", {"loop_factory": WepollEventLoop}), id="asyncio[WepollEventLoop]")]
)
def anyio_backend(request: pytest.FixtureRequest):
    return request.param

@pytest.mark.anyio
async def test_dns_resolver_over_wepoll(anyio_backend):
    async with DNSResolver(["8.8.8.8", "8.8.4.4"], event_thread=False) as dns:
        assert await dns.query("google.com", "A") 





import sys
from select import select
from socket import socket
from types import TracebackType
from typing import final

if sys.version_info < (3, 11):
    from typing_extensions import Self
else:
    from typing import Self

@final
class epoll:
    def __init__(self, sizehint: int = ...) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None = None,
        exc_value: BaseException | None = ...,
        exc_tb: TracebackType | None = None,
        /,
    ) -> None: ...
    def close(self) -> None: ...
    closed: bool
    def fileno(self) -> int: ...
    def register(self, fd: int | socket, eventmask: int = ...) -> None: ...
    def modify(self, fd: int | socket, eventmask: int) -> None: ...
    def unregister(self, fd: int | socket) -> None: ...
    def poll(
        self, timeout: float | None = None, maxevents: int = -1
    ) -> list[tuple[int, int]]: ...

    # Maybe in a Future release but we shall see...
    @classmethod
    def fromfd(cls, fd: int, /) -> epoll:
        """creates a epoll (wepoll) from msvcrt using get_osfhandle(fd) this may be upgraded in the future
        to use an optimized version or a 1:1 derrived copy of the CPython code for more speed."""
        



import math
import sys
from selectors import _PollLikeSelector as PollLikeSelector
from typing import TYPE_CHECKING, Optional

from ._wepoll import epoll
from .flags import EPOLLIN, EPOLLOUT

# Added here if you didn't want to grab from somewhere else
EVENT_READ = 1  # (1 << 0)
EVENT_WRITE = 2  # (1 << 1)

if TYPE_CHECKING:
    from selectors import SelectorKey

# 3.13 compatabaility flags
_NOT_EPOLLIN = ~EPOLLIN
_NOT_EPOLLOUT = ~EPOLLOUT


class EpollSelector(PollLikeSelector):
    """Wepoll-based selector for windows operating systems"""

    _selector_cls = epoll
    _EVENT_READ = EPOLLIN
    _EVENT_WRITE = EPOLLOUT

    if TYPE_CHECKING:
        _selector: epoll
        if sys.version_info < (3, 13):
            _fd_to_key: "dict[int, SelectorKey]"

    if sys.version_info < (3, 13):

        def select(self, timeout=None):
            # This is shared between poll() and epoll().
            # epoll() has a different signature and handling of timeout parameter.
            if timeout is None:
                timeout = None
            elif timeout <= 0:
                timeout = 0
            else:
                # NOTE: Our Poll does it by seconds so we get to ignore that
                pass

            ready = []
            try:
                fd_event_list = self._selector.poll(timeout)
            except InterruptedError:
                return ready
            for fd, event in fd_event_list:
                events = 0
                if event & ~self._EVENT_READ:
                    events |= EVENT_WRITE
                if event & ~self._EVENT_WRITE:
                    events |= EVENT_READ

                key = self._key_from_fd(fd)
                if key:
                    ready.append((key, events & key.events))
            return ready
    else:

        def select(self, timeout=None):
            if timeout is None:
                timeout = -1
            elif timeout <= 0:
                timeout = 0
            else:
                pass

            # epoll_wait() expects `maxevents` to be greater than zero;
            # we want to make sure that `select()` can be called when no
            # FD is registered.
            max_ev = len(self._fd_to_key) or 1

            ready = []
            try:
                fd_event_list = self._selector.poll(timeout, max_ev)
            except InterruptedError:
                return ready

            fd_to_key = self._fd_to_key
            for fd, event in fd_event_list:
                key = fd_to_key.get(fd)
                if key:
                    events = (event & _NOT_EPOLLIN and EVENT_WRITE) | (
                        event & _NOT_EPOLLOUT and EVENT_READ
                    )
                    ready.append((key, events & key.events))
            return ready



from Cython.Build import cythonize
from setuptools import Extension, setup

import sys
if sys.version_info[:2] < (3, 10):
    from warnings import warn
    warn(
        "3.9 will be dropped in"
        "version 0.1.5 consider upgrading",
        DeprecationWarning
    )


# TODO: Move to pyproject.toml setup like in pyduktape3
setup(
    ext_modules=cythonize(
        [
            Extension(
                "wepoll._wepoll",
                ["wepoll/_wepoll.pyx", "vendor/wepoll/wepoll.c"],
                include_dirs=["vendor/wepoll"],
                libraries=["advapi32", "iphlpapi", "psapi", "ws2_32"],
            )
        ]
    )
)




import selectors
import tempfile
import unittest
from test.test_selectors import ScalableSelectorMixIn, BaseSelectorTestCase
from wepoll import EpollSelector

# Code is borrowed from python's testsuite ensure wepoll matches up with unix epolls

class EpollSelectorTestCase(BaseSelectorTestCase, ScalableSelectorMixIn,
                            unittest.TestCase):

    SELECTOR = EpollSelector
    def test_modify_unregister(self):
        raise self.skipTest("")

    def test_register_file(self):
        # epoll(7) returns EPERM when given a file to watch
        s = self.SELECTOR()
        with tempfile.NamedTemporaryFile() as f:
            with self.assertRaises(IOError):
                s.register(f, selectors.EVENT_READ)
            # the SelectorKey has been removed
            with self.assertRaises(KeyError):
                s.get_key(f)

